%  Set parameters for LCMV beamformer inverse solution.
%  
% - required field 
% megfile, basisfile, brainfile, currfile, twin_meg, twin_noise
%
% ***
% *** Basic Parameters
% ***
% ******************************
% - .megfile (.meg.mat)
%     MEG data file to be analyzed. This is given by cell array of strings.
%    e.g. megfile = {'SESSION1.meg.mat','SESSION2.meg.mat'}
%
% - .basisfile (.basis.mat)
%     Lead field file for the high resolution model.
%    e.g. megfile = {'SESSION1.basis.mat','SESSION2.basis.mat'}
%
% - .brainfile (.brain.mat)
%     Brain file. 
%
% - .currfile (.curr.mat)
%     Result file. Estimated currents will be saved into this file. 
%
% - .twin_meg
%    Time window of meg data in .megfile used for analysis. This window 
%   is covered by several small time windows specfied by .Tperiod and
%   .Tnext. It is specified by the start and end of the sampling frame, 
%   not by the actual measurement time. 
%   e.g. twin_meg = [1 600]  
%
% - .twin_noise
%   Time window (start and end) used for estimation 
%   of the covariance of the observation noise
%
%
%
% ***
% *** Advanced Parameters
% ***
% ******************************
%
% - .actfile (.act.mat)
%    Activity map file. 
%
% - .areafile (.area.mat)
%    Area file. 
%
% - .act_key
%    ID of an activity map in .actfile. The specified activity map is 
%   used as the prior information to VB variance estimation.
%
% - .area_key
%    ID of an area in .areafile. Possible (focal) dipoles location are 
%   restricted to the specified area. 
%
% - .noise_model
%   Specifying noise model. 
%   -- 1 spherical, same noise variance for all sensors
%   -- 2 ellipsoidal, different noise variances but independent
%   -- 3 full covariance, most general noise model 
%
%
% - .noise_reg (0.01)
%   Regularization parameter for the normalized covariance matrix
%   of the additive noise. This value should be determined
%   considering that the mean of the diagonal part of the
%   normalized covariance matrix is 1.
%
% 2016/01/27 O.Yamashita
% * remove unnecessary parameters 
% 2015/08/21 O.Yamahita

function lcmv_parm = set_lcmv_parm(varargin)

parm = finputcheck(varargin, ...
              { 'brainfile'     'string'   []       []; ...
                'megfile'       'string'   []       []; ...
                'basisfile'     'string'   []       [];...
                'currfile'      'string'   []       [];...
                'twin_meg'      'integer'  []       [];...
                'twin_noise'    'integer'  []       [];...
                'noise_model'   'integer'  [1 2 3]   3;...
                'noise_reg'     'real'     []       0.01;...
                'areafile'      'string'   []       '';...
                'actfile'       'string'   []       '';...
                'area_key'      'string'   []      'Cortex';...
                'act_key'       'string'   []      'Uniform';...
                'patch_norm'    'boolean'  []      ON;...
                'trial_average' 'boolean'  []      OFF;...
                'saveifilt'    'boolean'   []      ON;...
                });

if isstr(parm), error(parm);end

lcmv_parm = parm;

