%  Set parameters for LORETA inverse solution.  
%
% - Example
% lor_parm = set_loreta_parm2('brainfile',brainfile,...
%     'megfile',megfile,...
%     'basisfile',basisfile,...
%     'laplacianfile', [rdir 'lap.mat'],...
%     'areafile',areafile,...
%     'actfile', actfile,...
%     'twin_meg', twin_meg,...
%     'twin_noise',twin_noise,...
%     'currfile', 'loreta.curr.mat');
%
% - required field 
% megfile, basisfile, brainfile, currfile, twin_meg, twin_noise
% laplacianfile
%
% ***
% *** Basic Parameters
% ***
% ******************************
% - .megfile (.meg.mat)
%     MEG data file to be analyzed. This is given by cell array of strings.
%    e.g. megfile = {'SESSION1.meg.mat','SESSION2.meg.mat'}
%
% - .basisfile (.basis.mat)
%     Lead field file for the high resolution model.
%    e.g. megfile = {'SESSION1.basis.mat','SESSION2.basis.mat'}
%
% - .brainfile (.brain.mat)
%     Brain file. 
%
% - .currfile (.curr.mat)
%     Result file. Estimated currents will be saved into this file. 
%
% - .twin_meg
%    Time window of meg data in .megfile used for analysis. This window 
%   is covered by several small time windows specfied by .Tperiod and
%   .Tnext. It is specified by the start and end of the sampling frame, 
%   not by the actual measurement time. 
%   e.g. twin_meg = [1 600]  
%
% - .twin_noise
%   Time window (start and end) used for estimation 
%   of the covariance of the observation noise
%
% - .lanplacianfile
%    Laplacian file.
%
% - .laplacian_mode
%   When specified 'calc', Laplacian matrix is calculated but not saved
%   (.laplacianfile is ignored).
%   When specified 'save', Laplacian matrix is calculated and the result is
%   saved in .laplacianfile.
%   When specified 'skip', calculation of Laplacian matrix is skipped 
%   (.laplacianfile is ignored).
%   When specified 'load', Laplacian matrix is loaded from .laplacianfile 
%  (therefore you have to have calulated Laplacian matrix onece).
%
% ***
% *** Advanced Parameters
% ***
% ******************************
%
% - .actfile (.act.mat)
%    Activity map file. If specified, weighted minimum norm solution is
%    obtained.
%
% - .areafile (.area.mat)
%    Area file. If specified, current sources are estimated in the area.
%
% - .act_key
%    ID of an activity map in .actfile. The specified activity map is 
%   used as the prior information to VB variance estimation.
%
% - .area_key
%    ID of an area in .areafile. Possible (focal) dipoles location are 
%   restricted to the specified area. 
%
% - .noise_model
%   Specifying noise model. 
%   -- 1 spherical, same noise variance for all sensors
%   -- 2 ellipsoidal, different noise variances but independent
%   -- 3 full covariance, most general noise model  
%
% - .noise_reg (0.01)
%   Regularization parameter for the normalized covariance matrix
%   of the additive noise. This value should be determined
%   considering that the mean of the diagonal part of the
%   normalized covariance matrix is 1.
%
% - .cosval (0.5)
%   Neighbor vertices which satisfies this condition are used to compute a laplacian matrix   
%
% 2015/08/24 O.Yamashita
function loreta_parm = set_loreta_parm(varargin)


parm = finputcheck(varargin, ...
              { 'brainfile'     'string'   []       []; ...
                'megfile'       'string'   []       []; ...
                'basisfile'     'string'   []       [];...
                'currfile'      'string'   []       [];...
                'laplacianfile' 'string'   []       [];...
                'twin_meg'      'integer'  []       [];...
                'twin_noise'    'integer'  []       [];...
                'laplacian_mode' 'string'  {'calc','load','save','skip'}  'save';...
                'cosval'        'real'     []       0.5;...
                'noise_model'   'integer'  [1 2 3]   3;...
                'noise_reg'     'real'     []       0.01;...
                'areafile'      'string'   []       '';...
                'actfile'       'string'   []       '';...
                'area_key'      'string'   []      'Cortex';...
                'act_key'       'string'   []      'Uniform';...
                'patch_norm'    'boolean'  []      ON;...
                'temporal_filter' 'boolean'  []    OFF;...
                'criteria'     'string'  {'ABIC','GCV'}   'ABIC';...
                'npoints'      'integer'   []      100;...
                'isplot'       'boolean'   []      ON;...
                'saveifilt'    'boolean'   []      ON;...
               } );

if isstr(parm), error(parm);end


loreta_parm = parm;


