function [IF] = wmn_calc_invfilter(G, X, w, lam, laplacian)
% Calculate inverse filter using the optimized regularization parameter 
%  
% Solving the following minimization problem  
%   min {E(J) = (B-GJ)'X(B-GJ) + lam*J'WJ}  
% gives
% J = (lam*W)^(-1) G'(G(lam*W)^(-1)G'+X^(-1))^(-1) * B .
%
% When laplacian exists, solving the following minimization problem
%   min {E(J) = (B-GJ)'X(B-GJ) + lam*J'LWLJ}  
% gives 
% J = inv(L)*(lam*W)^(-1) H'(H(lam*W)^(-1)H'+X^(-1))^(-1) * B .
% 
% H = G*L^(-1)
% 
% ---- syntax 
% [IF] = wmn_calc_invfilter(G, X, w, lam)  : minimum norm 
% [IF] = wmn_calc_invfilter(G, X, w, lam, laplacian) : LORETA
%
% ---- output
% IF : inverse filter
%
% ---- input
% G : lead field matrix  (Nch * Nj matrix)
% X : observation weight matrix (inverse of observation noise)
% w : the diagonal elements of prior weight matrix W (should be diagonal, to avoid out of memory) 
% lam : optimal regularization parameter given by "wmn_solution"
% lapacian : if exist, LORETA inverse filter is calculated.
%
% 2007-01-30 OY 

if nargin < 5
    H = G;
else
    H = G/laplacian;
end

lw = (1/lam) * (1./w);    % 1/lam*W^(-1)
lwH = repmat(lw, [1,size(H,1)]) .* H';     % lw*H'

IF = lwH * inv(H*lwH+inv(X));

if nargin == 5,
    IF = laplacian\IF;
end
